<?xml version="1.0" encoding="utf-8"?>

  <!--
    This stylesheet transform openXML into PS Format
  
    @author Christine Feng 
    @copyright Allette Systems Pty Ltd 
    @param media
    @param rels
    @param numbering
    @param docprops
    @param footnotes
  -->
  
<xsl:stylesheet version="2.0"
								xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
								xmlns:ve="http://schemas.openxmlformats.org/markup-compatibility/2006"
								xmlns:o="urn:schemas-microsoft-com:office:office"
								xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships"
								xmlns:m="http://schemas.openxmlformats.org/officeDocument/2006/math"
								xmlns:v="urn:schemas-microsoft-com:vml"
								xmlns:wp="http://schemas.openxmlformats.org/drawingml/2006/wordprocessingDrawing"
								xmlns:w10="urn:schemas-microsoft-com:office:word"
								xmlns:w="http://schemas.openxmlformats.org/wordprocessingml/2006/main"
								xmlns:wne="http://schemas.microsoft.com/office/word/2006/wordml"
								xmlns:pic="http://schemas.openxmlformats.org/drawingml/2006/picture"
								xmlns:a="http://schemas.openxmlformats.org/drawingml/2006/main"
								xmlns:rs="http://schemas.openxmlformats.org/package/2006/relationships"
								xmlns:cp="http://schemas.openxmlformats.org/package/2006/metadata/core-properties"
							  xmlns:dc="http://purl.org/dc/elements/1.1/"
							  xmlns:dcterms="http://purl.org/dc/terms/"
								xmlns:ps="http://www.pageseeder.com/editing/2.0"
								xmlns:xs="http://www.w3.org/2001/XMLSchema"
								exclude-result-prefixes="ve o r m v wp w10 w wne pic a rs cp dc dcterms xs ps">
	
	<xsl:param name="media" />
	<xsl:param name="rels" />
	<xsl:param name="numbering" />
	<xsl:param name="docprops" />
  <xsl:param name="footnotes" />
  
	<xsl:param name="relsdoc"      select="document(replace($rels,     '\\', '/'))" />
	<xsl:param name="numberingdoc" select="document(replace($numbering,'\\', '/'))" />
	<xsl:param name="docpropsdoc"  select="document(replace($docprops, '\\', '/'))" />
	<xsl:param name="footnotesdoc" select="document(replace($footnotes,'\\', '/'))" />
	
	<xsl:decimal-format name="digits" decimal-separator="." grouping-separator="," />

	<xsl:output method="xml" version="1.0" indent="no" encoding="UTF-8" />
    <xsl:strip-space elements="*"/>
    	
	<xsl:template match="/">
		<xsl:apply-templates select="w:document" />
	</xsl:template>
  
  <!--##root##-->
  <!--##properties##-->
  <!--##toc##-->
	<xsl:template match="w:document">
		<xsl:processing-instruction name="stylesheet">
			<xsl:text>format="standard"</xsl:text>
		</xsl:processing-instruction>
		<root>
		  <xsl:if test="doc-available($docprops)">
		    <properties>
          <xsl:for-each select="$docpropsdoc/cp:coreProperties/*">
            <property name="{local-name()}"><xsl:value-of select="."/></property>
          </xsl:for-each>
        </properties>
		  </xsl:if>
      <section id="title">
        <body/>
      </section>
      <section id="toc">
        <toc/>
      </section>
			<xsl:apply-templates select="w:body" />
		</root>
	</xsl:template>
	
<!-- =============================================================
         Match w:body
         Sections are created for every level of headings
     ============================================================= -->
	<!--##section##-->
	<!--##body##-->
	<xsl:template match="w:body">
	
		<xsl:for-each-group select="*" group-starting-with="w:p[matches(w:pPr/w:pStyle/@w:val, 'Heading(1|2|3|4|5|6)')][string-join(w:r//text(), '') != '']">
			<section id="{count(preceding::w:p)+1}">
				<body>
					<xsl:apply-templates select="current-group()" />
				</body>
			</section>
		</xsl:for-each-group>
	</xsl:template>
  
  
	<xsl:template match="w:p">
		<xsl:variable name="styleName">
			<xsl:value-of select="w:pPr/w:pStyle/@w:val" />
		</xsl:variable>
		<xsl:choose>
      <!-- strip out empty headings -->
      <xsl:when test="matches($styleName, 'Heading(1|2|3|4|5|6)') and string-join(w:r//text()|w:hyperlink//text(), '') = ''" />
      <!-- when headings not in tables -->
      <!--##heading##-->
      <xsl:when test="matches($styleName, 'Heading(1|2|3|4|5|6)') and not(ancestor::w:tbl)">
        <xsl:element name="heading{substring-after($styleName, 'Heading')}">
          <xsl:if test="w:pPr/w:numPr/w:numId">
            <xsl:attribute name="numbered">true</xsl:attribute>
          </xsl:if>
          <xsl:apply-templates select="w:r|w:hyperlink">
            <xsl:with-param name="inHeading" select="true()" />
          </xsl:apply-templates>
        </xsl:element>
      </xsl:when>
			<!-- when list -->
			<!--##list##-->
			<xsl:when test="w:pPr/w:numPr[w:ilvl][w:numId]">
			    <xsl:variable name="listID" select="w:pPr/w:numPr/w:numId/@w:val"/>
				
			    <xsl:choose>
			    	<!-- when already processed in list section do nothing -->
            <!-- JB: changed to match previous p element with any list level, not just current list level -->
				    <xsl:when test="preceding-sibling::w:p[1][./w:pPr/w:numPr/w:numId/@w:val]"/>
					
				    <xsl:otherwise>
						<xsl:call-template name="list">
							<xsl:with-param name="abstractId" select="$numberingdoc/w:numbering/w:num[@w:numId=$listID]/w:abstractNumId/@w:val"/>
							<xsl:with-param name="level" select="w:pPr/w:numPr/w:ilvl/@w:val"/>
						</xsl:call-template>
				    </xsl:otherwise>
			    </xsl:choose>
			</xsl:when>
			<xsl:otherwise>
				<para>
					<!--
						w:p with a style name is transformed into a para with a type attribute
						this attribute can be used to apply styles in ps-to-openXML by turning on $take-style-from-type-attribute
            JB: removed because it is not valid according to standard.xsd
          <xsl:if test="$styleName!=''">
            <xsl:attribute name="type">
              <xsl:value-of select="$styleName" />
            </xsl:attribute>
          </xsl:if>
					-->
					<xsl:apply-templates select="w:r" />
				</para>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

<!-- =============================================================
         Match w:p with w:numPr
         create list item
     ============================================================= -->
    <xsl:template match="w:p" mode="insideList">
	    <xsl:variable name="level" select="w:pPr/w:numPr/w:ilvl/@w:val"/>
    	
	    <xsl:variable name="nested-list" select="following-sibling::w:p[1][w:pPr/w:numPr/w:ilvl/@w:val &gt; $level]" />
      <!--##item##-->
	    <item>
		    <xsl:apply-templates/>
  			<xsl:for-each select="$nested-list">
  			  <xsl:variable name="nlistID" select="w:pPr/w:numPr/w:numId/@w:val"/>
  				<xsl:call-template name="list">
  					<xsl:with-param name="abstractId" select="$numberingdoc/w:numbering/w:num[@w:numId=$nlistID]/w:abstractNumId/@w:val"/>
  					<xsl:with-param name="level" select="w:pPr/w:numPr/w:ilvl/@w:val"/>
  				</xsl:call-template>
  			</xsl:for-each>
	    </item>
	    
  		<xsl:variable name="listID" select="w:pPr/w:numPr/w:numId/@w:val"/>
  		<xsl:choose>
  			<xsl:when test="$nested-list">
  				<!-- skip to next item at this level just after nested list -->
  				<xsl:apply-templates select="following-sibling::w:p
  						[w:pPr/w:numPr/w:ilvl/@w:val=$level][w:pPr/w:numPr/w:numId/@w:val=$listID][1]
  						[preceding-sibling::w:p[1]/w:pPr/w:numPr/w:ilvl/@w:val &gt; $level]" mode="insideList">
  				</xsl:apply-templates>
  			</xsl:when>
  			<xsl:otherwise>
  				<!-- get next item if it is at the same level -->
  				<xsl:apply-templates select="following-sibling::w:p[1]
  						[w:pPr/w:numPr/w:ilvl/@w:val=$level][w:pPr/w:numPr/w:numId/@w:val=$listID]" mode="insideList">
  				</xsl:apply-templates>
  			</xsl:otherwise>
  		</xsl:choose>
    </xsl:template>
    
<!-- =============================================================
         create list or nlist, based on the list type defined
         in numbering.xml
     ============================================================= -->
  <!--##list##-->
  <!--##nlist##-->
  <xsl:template name="list">
    <xsl:param name="abstractId" />
    <xsl:param name="level"/>
		<xsl:comment>level = <xsl:value-of select="$level"/></xsl:comment>
    <xsl:comment>abstractId= <xsl:value-of select="$abstractId"/>
    </xsl:comment>
		<xsl:choose>
			<xsl:when test="$numberingdoc/w:numbering/w:abstractNum[@w:abstractNumId=$abstractId]/w:lvl[@w:ilvl=$level]/w:numFmt/@w:val='bullet'">
				<list>
					<xsl:apply-templates select="." mode="insideList"/>
				</list>
			</xsl:when>
			<xsl:otherwise>
				<nlist>
        <!-- JB: removed because it's not valid according to standard.xsd
					<xsl:attribute name="liststyle">
						<xsl:value-of select="$numberingdoc/w:numbering/w:abstractNum[@w:abstractNumId=$abstractId]/w:lvl[@w:ilvl=$level]/w:numFmt/@w:val"/>
					</xsl:attribute>
         -->
					<xsl:apply-templates select="." mode="insideList"/>
				</nlist>
			</xsl:otherwise>
		</xsl:choose>
  </xsl:template>


	<xsl:template match="w:r">
    <xsl:param name="inHeading" select="false()" />
		<xsl:choose>
			<xsl:when test="w:br | w:t">
				<xsl:call-template name="apply-wr-style">
					<xsl:with-param name="style"     select="w:rPr/w:rStyle/@w:val" />
					<xsl:with-param name="text"      select="w:br | w:t" />
					<xsl:with-param name="bold"      select="if (w:rPr/w:b) then 'true' else 'false'" />
					<xsl:with-param name="italic"    select="if (w:rPr/w:i) then 'true' else 'false'" />
					<xsl:with-param name="underline" select="if (w:rPr/w:u) then 'true' else 'false'" />
					<xsl:with-param name="sub"       select="if (w:rPr/w:vertAlign[@w:val='subscript']) then 'true' else 'false'" />
					<xsl:with-param name="sup"       select="if (w:rPr/w:vertAlign[@w:val='superscript']) then 'true' else 'false'" />
				</xsl:call-template>
			</xsl:when>
			<!--##graphic##-->
			<xsl:when test="w:drawing and not($inHeading)">
				<xsl:variable name="rid" select="w:drawing//a:graphic/a:graphicData/pic:pic/pic:blipFill/a:blip/@r:embed" />
				<xsl:variable name="target" select="$relsdoc/rs:Relationships/rs:Relationship[@Id=$rid]/@Target" />
				<xsl:variable name="alt" select="substring-after($target,'media/')" />
				<xsl:variable name="height" select="if (w:drawing/wp:inline/wp:extent/@cy) then 
				                                   (number(w:drawing/wp:inline/wp:extent/@cy) idiv 9525) else 0"/>
				<xsl:variable name="width"  select="if (w:drawing/wp:inline/wp:extent/@cx) then 
                                           (number(w:drawing/wp:inline/wp:extent/@cx) idiv 9525) else 0"/>
				<graphic src="{concat($media,substring-after($target,'media'))}" alt="{$alt}">
          <xsl:if test="$width!=0"><xsl:attribute name="width" select="$width"/></xsl:if>
          <xsl:if test="$height!=0"><xsl:attribute name="height" select="$height"/></xsl:if>
				</graphic>
			</xsl:when>
			<xsl:when test="w:pict and not($inHeading)">
				<!--##caption##-->
				<xsl:for-each select="w:pict//w:txbxContent/w:p[w:pPr/w:pStyle/@w:val='Caption']">
					<paraLabel name="caption">
						<xsl:value-of select="w:r/w:t" />
						<xsl:value-of select="w:fldSimple/w:r/w:t" />
					</paraLabel>
				</xsl:for-each>
				<!--##graphic##-->
				<xsl:if test="w:pict//v:shape/v:imagedata/@r:id">
					<xsl:variable name="rid" select="w:pict//v:shape/v:imagedata/@r:id" />
					<xsl:variable name="target" select="$relsdoc/rs:Relationships/rs:Relationship[@Id=$rid]/@Target" />
					<xsl:variable name="alt" select="substring-after($target,'media/')" />
					<graphic src="{concat($media,substring-after($target,'media'))}" alt="{$alt}" />
				</xsl:if>
			</xsl:when>
			<xsl:when test="doc-available($footnotes) and w:footnoteReference">
			  <sup>
          <xref frag="default" display="document" type="None" reverselink="true" reversetitle="" reversetype="None" href="footnotes/{w:footnoteReference/@w:id}.xml">
            <xsl:value-of select="w:footnoteReference/@w:id"/>
          </xref>
        </sup>
      </xsl:when>
		</xsl:choose>
	</xsl:template>
	
<!-- =============================================================
        calls itself recursively to create inline elements
     ============================================================= -->
	<xsl:template name="apply-wr-style">
		<!-- This template processes inline elements recursively -->
		<xsl:param name="style" />
		<xsl:param name="text" />
		<xsl:param name="bold" />
		<xsl:param name="italic" />
		<xsl:param name="underline" />
		<xsl:param name="sub" />
		<xsl:param name="sup" />
		<xsl:choose>
			<xsl:when test="$style!=''">
				<!-- inlineLabel type="{$style}"-->
					<xsl:call-template name="apply-wr-style">
						<xsl:with-param name="style" select="''" />
						<xsl:with-param name="text" select="$text" />
						<xsl:with-param name="bold" select="'false'" />
						<xsl:with-param name="italic" select="$italic" />
						<xsl:with-param name="underline" select="$underline" />
						<xsl:with-param name="sub" select="$sub" />
						<xsl:with-param name="sup" select="$sup" />
					</xsl:call-template>
				<!-- /inlineLabel-->
			</xsl:when>
			<xsl:when test="$bold='true'">
			   <!--##bold##-->
				<bold>
					<xsl:call-template name="apply-wr-style">
						<xsl:with-param name="style" select="''" />
						<xsl:with-param name="text" select="$text" />
						<xsl:with-param name="bold" select="'false'" />
						<xsl:with-param name="italic" select="$italic" />
						<xsl:with-param name="underline" select="$underline" />
						<xsl:with-param name="sub" select="$sub" />
						<xsl:with-param name="sup" select="$sup" />
					</xsl:call-template>
				</bold>
			</xsl:when>
			<xsl:when test="$italic='true'">
				<!--##italic##-->
				<italic>
					<xsl:call-template name="apply-wr-style">
						<xsl:with-param name="style" select="''" />
						<xsl:with-param name="text" select="$text" />
						<xsl:with-param name="bold" select="$bold" />
						<xsl:with-param name="italic" select="'false'" />
						<xsl:with-param name="underline" select="$underline" />
						<xsl:with-param name="sub" select="$sub" />
						<xsl:with-param name="sup" select="$sup" />
					</xsl:call-template>
				</italic>
			</xsl:when>
			<xsl:when test="$underline='true'">
				<!--##underline##-->
				<underline>
					<xsl:call-template name="apply-wr-style">
						<xsl:with-param name="style" select="''" />
						<xsl:with-param name="text" select="$text" />
						<xsl:with-param name="bold" select="$bold" />
						<xsl:with-param name="italic" select="$italic" />
						<xsl:with-param name="underline" select="'false'" />
						<xsl:with-param name="sub" select="$sub" />
						<xsl:with-param name="sup" select="$sup" />
					</xsl:call-template>
				</underline>
			</xsl:when>
			<xsl:when test="$sub='true'">
				<!--##sub##-->
				<sub>
					<xsl:call-template name="apply-wr-style">
						<xsl:with-param name="style" select="''" />
						<xsl:with-param name="text" select="$text" />
						<xsl:with-param name="bold" select="$bold" />
						<xsl:with-param name="italic" select="$italic" />
						<xsl:with-param name="underline" select="$underline" />
						<xsl:with-param name="sub" select="'false'" />
						<xsl:with-param name="sup" select="$sup" />
					</xsl:call-template>
				</sub>
			</xsl:when>
			<xsl:when test="$sup='true'">
				<!--##sup##-->
				<sup>
					<xsl:call-template name="apply-wr-style">
						<xsl:with-param name="style" select="''" />
						<xsl:with-param name="text" select="$text" />
						<xsl:with-param name="bold" select="$bold" />
						<xsl:with-param name="italic" select="$italic" />
						<xsl:with-param name="underline" select="$underline" />
						<xsl:with-param name="sub" select="$sub" />
						<xsl:with-param name="sup" select="'false'" />
					</xsl:call-template>
				</sup>
			</xsl:when>
			<xsl:otherwise>
				<xsl:for-each select="$text">
          <xsl:choose>
            <!--##br##-->
            <xsl:when test="self::w:br"><br /></xsl:when>
            <xsl:otherwise><xsl:value-of select="." /> </xsl:otherwise>
          </xsl:choose>
        </xsl:for-each>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- =============================================================
             match w:tbl
             table style is turned into @type
         ============================================================= -->
	<xsl:template match="w:tbl">
    <!--##table##-->

		<!-- xsl:apply-templates select="w:tr"/-->
		<table>
			<xsl:if test="w:tblPr/w:tblStyle/@w:val!=''">
				<!-- if the table has a style name, create the style attribute -->
				<xsl:attribute name="role">
    			<xsl:value-of select="w:tblPr/w:tblStyle/@w:val" />
    		</xsl:attribute>
			</xsl:if>
			<xsl:if test="count(w:tr) = 0">
			 <row>
			   <cell/>
			 </row>			
			</xsl:if>
			<xsl:apply-templates select="w:tblGrid" />
			<xsl:apply-templates select="w:tr" />
		</table>
	</xsl:template>

	<xsl:template match="w:tblGrid">
		<tgroup>
			<xsl:apply-templates select="w:gridCol" />
		</tgroup>
	</xsl:template>

	<xsl:template match="w:gridCol">
		<xsl:variable name="width"
			select="format-number(@w:w div 1440, '#,###.00', 'digits')" />
		<colspec colwidth="{concat($width,'in')}" />
	</xsl:template>

	<xsl:template match="w:tr">
    <!--##row##-->
		<row>
			<xsl:choose>
				<xsl:when test="w:trPr/w:tblHeader">
				<!-- when repeat table header is on, this row should contain hcells -->
					<xsl:apply-templates select="w:tc" mode="hcell"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:apply-templates select="w:tc"/>
				</xsl:otherwise>
			</xsl:choose>
			<xsl:if test="count(w:tc) = 0">
         <cell/>  
      </xsl:if>
		</row>
	</xsl:template>

	<xsl:template match="w:tc">
    <!--##cell##-->
    <cell>
			<xsl:if test="w:tcPr/w:gridSpan">
				<xsl:attribute name="colspan" select="w:tcPr/w:gridSpan/@w:val" />
			</xsl:if>
			<xsl:apply-templates />
		</cell>
		<!-- else, no content or merged cells -->
	</xsl:template>
	
	<xsl:template match="w:tc" mode="hcell">
    <!--##hcell##-->
    <hcell>
			<xsl:if test="w:tcPr/w:gridSpan">
				<xsl:attribute name="colspan" select="w:tcPr/w:gridSpan/@w:val" />
			</xsl:if>
			<xsl:apply-templates />
		</hcell>
		<!-- else, no content or merged cells -->
	</xsl:template>

	<xsl:template match="w:hyperlink">
    <!--##link##-->
   <xsl:choose>
	   <xsl:when test="@r:id">
	     <xsl:variable name="rid" select="@r:id" />
		    <link
		      href="{$relsdoc/rs:Relationships/rs:Relationship[@Id=$rid]/@Target}">
		      <xsl:value-of select="w:r/w:t" />
		    </link>
	   </xsl:when>
	   <xsl:otherwise>
	     <xsl:apply-templates select="w:r"/>
	   </xsl:otherwise>
	 </xsl:choose>
		
	</xsl:template>

</xsl:stylesheet>