<?xml version="1.0"?> 
<xsl:stylesheet version="2.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:ps="http://www.pageseeder.com/editing/2.0"
                xmlns:psf="http://www.pageseeder.com/function"
                xmlns:dec="java:java.net.URLDecoder"
                exclude-result-prefixes="#all">
<!-- 
  Generate the documents ready to be copied or FTPed to a Berlioz website.
  All the paths (XRefs, Graphics) are modified to be relative.
 -->

<xsl:param name="input-base" />
<xsl:param name="output-base" />
<xsl:param name="siteprefix" select="''" />

<xsl:param name="group-folder" select="''"/>
<xsl:param name="single" select="''"/>
<xsl:param name="version-folder" select="''"/>
<xsl:param name="website-root" select="''"/>
<xsl:param name="website-def" select="''"/>

<xsl:variable name="group-folder-no-prefix" select="substring-after($group-folder,$siteprefix)" />
<xsl:variable name="website-folder" select="concat($group-folder,$website-root)" />
<xsl:variable name="website-content-root" select="concat($website-root,'/content')" />


<xsl:variable name="website-def-url" select="concat('file:///',replace($website-def, '\\', '/'))" />
<xsl:variable name="website-def-doc" select="if (doc-available($website-def-url)) then doc($website-def-url) else ()"/>

<xsl:output method="text"/>

<xsl:template match="/uris">
  <xsl:for-each select="uri[starts-with(behavior, 'standard')]">
    <xsl:variable name="doc" select="document(concat(replace($input-base, '\\', '/'), '/', decodedpath))" />
    <xsl:variable name="new-url" select="concat(replace($output-base, '\\', '/'), '/', decodedpath)" />
    <xsl:result-document href="{$new-url}" method="xml" indent="no">
      <xsl:apply-templates select="$doc">
        <xsl:with-param name="uri-folder" select="concat(string-join(tokenize(current()/path, '/')[position() != last()], '/'), '/')" tunnel="yes" />
      </xsl:apply-templates>
    </xsl:result-document>
    <xsl:value-of select="$new-url" /><xsl:text>
</xsl:text>
  </xsl:for-each>
</xsl:template>

<xsl:template match="*">
  <xsl:copy>
    <xsl:copy-of select="@*"/>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<xsl:template match="xref|blockXref|ps:reversexref">
  <xsl:variable name="name" select="local-name()"/>
  
  <xsl:element name="{$name}">
    <xsl:copy-of select="@*[not(local-name()='href')]" />
    <!-- anchor -->
    <xsl:variable name="anchor" select="
       if (contains(@href,'#'))
        then concat('#',substring-after(@href,'#psFrag'))
       else ''
    ">
    </xsl:variable>
    <xsl:variable name="link" select="
       if (contains(@href,'?'))
        then substring-before(@href,'?')
       else @href
    ">
    </xsl:variable>
    <xsl:variable name="href-no-group" select="concat($version-folder,substring-after($link,$website-folder),$anchor)"/>
    <xsl:choose>
      <!-- link to another group -->
      <xsl:when test="not(starts-with($link,$website-folder))">
        <xsl:variable name="mapping" select="$website-def-doc//section[@id='xrefmapping-config']//paraLabel[@name='XRef_Mapping']"/>
        <xsl:variable name="new-href">
          <xsl:for-each select="$mapping">
            <xsl:variable name="ps-path" select=".//inlineLabel[@name='PS_Path']"/>
            <xsl:variable name="website-path" select=".//inlineLabel[@name='Website_Path']"/>
            <xsl:if test="starts-with($link,$ps-path)">
              <href><xsl:value-of select="concat($website-path,substring-after($link,$ps-path),$anchor)"/></href>
            </xsl:if>
          </xsl:for-each>
        </xsl:variable>
        <xsl:choose>
          <!-- when mapping found, use it -->
          <xsl:when test="$new-href/href">
            <xsl:attribute name="href" select="$new-href/href[1]"/>
          </xsl:when>
          <!-- when contains current website root/content, remove it -->
          <xsl:when test="contains($link,$website-content-root)">
            <xsl:attribute name="href" select="concat('/content',substring-before(substring-after($link,$siteprefix),$website-content-root),substring-after($link,$website-content-root),$anchor)"/>
          </xsl:when>
          <!-- when contains current website root, remove it -->
          <xsl:when test="contains($link,$website-root)">
            <xsl:attribute name="href" select="concat(substring-before(substring-after($link,$siteprefix),$website-root),substring-after($link,$website-root),$anchor)"/>
          </xsl:when>
          <!-- otherwise just remove siteprefix -->
          <xsl:otherwise>
            <xsl:attribute name="href" select="concat(substring-after($link,$siteprefix),$anchor)"/>            
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
	    <!-- otherwise internal group link -->
	    <xsl:otherwise>
	      <xsl:attribute name="href" select="psf:resolve-website-path(@href,$website-root,$version-folder,$single,$group-folder,$siteprefix)"/>
	    </xsl:otherwise>
    </xsl:choose>
    <xsl:apply-templates/>
  </xsl:element>
</xsl:template>

<xsl:template match="ps:uri">
  <ps:uri>
    <xsl:copy-of select="@*[not(local-name()='path' or local-name()='decodedpath')]" />
    <xsl:if test="not(path)">
	    <xsl:attribute name="path" select="psf:resolve-website-path(@path,$website-root,$version-folder,$single,$group-folder,$siteprefix)"/>
	    <xsl:attribute name="decodedpath" select="psf:resolve-website-path(@decodedpath,$website-root,$version-folder,$single,$group-folder,$siteprefix)"/>
    </xsl:if>
    <xsl:apply-templates />
  </ps:uri> 
</xsl:template>

<!-- for backward compatibility with old xsl-documentInfo parameter -->
<xsl:template match="path">
  <path>
    <xsl:value-of select="psf:resolve-website-path(.,$website-root,$version-folder,$single,$group-folder,$siteprefix)"/>
  </path> 
</xsl:template>

<!-- for backward compatibility with old xsl-documentInfo parameter -->
<xsl:template match="decodedpath">
  <xsl:variable name="link" select="."/>
  <decodedpath>
    <xsl:choose>
    <!-- link to another group -->
    <xsl:when test="not(starts-with($link,$website-folder))">
      <xsl:variable name="mapping" select="$website-def-doc//section[@id='xrefmapping-config']//paraLabel[@name='XRef_Mapping']"/>
      <xsl:variable name="new-href">
        <xsl:for-each select="$mapping">
          <xsl:variable name="ps-path" select="dec:decode(.//inlineLabel[@name='PS_Path'],'UTF-8')"/>
          <xsl:variable name="website-path" select="dec:decode(.//inlineLabel[@name='Website_Path'],'UTF-8')"/>
          <xsl:if test="starts-with($link,$ps-path)">
            <href><xsl:value-of select="concat($website-path,substring-after($link,$ps-path))"/></href>
          </xsl:if>
        </xsl:for-each>
      </xsl:variable>
      <xsl:choose>
        <!-- when mapping found, use it -->
        <xsl:when test="$new-href/href">
          <xsl:value-of select="$new-href/href[1]"/>
        </xsl:when>
        <!-- when contains current website root/content, remove it -->
        <xsl:when test="contains($link,$website-content-root)">
          <xsl:value-of select="concat('/content',substring-before(substring-after($link,$siteprefix),$website-content-root),substring-after($link,$website-content-root))"/>
        </xsl:when>
        <!-- when contains current website root, remove it -->
        <xsl:when test="contains($link,$website-root)">
          <xsl:value-of select="concat(substring-before(substring-after($link,$siteprefix),$website-root),substring-after($link,$website-root))"/>
        </xsl:when>
        <!-- otherwise just remove siteprefix -->
        <xsl:otherwise>
          <xsl:value-of select="substring-after($link,$siteprefix)"/>            
        </xsl:otherwise>
      </xsl:choose>
    </xsl:when>
    <!-- otherwise internal group link -->
    <xsl:otherwise>
      <xsl:value-of select="psf:resolve-website-path($link,$website-root,$version-folder,$single,$group-folder,$siteprefix)"/>
    </xsl:otherwise>
    </xsl:choose>
  </decodedpath>
  
</xsl:template>

<xsl:template match="graphic">
  <xsl:param name="uri-folder"    tunnel="yes" />
  
  <graphic>
    <xsl:attribute name="src">
	    <xsl:choose>
      <xsl:when test="starts-with(@src,'http://')">
        <xsl:value-of select="@src"/>      
      </xsl:when>
	    <xsl:when test="starts-with(@src,concat($siteprefix,'/uri/'))">
        <xsl:value-of select="psf:resolve-website-path(@src,$website-root,$version-folder,$single,$group-folder,$siteprefix)"/>
	    </xsl:when>
	    <xsl:otherwise>
	      <xsl:variable name="path" select="if (starts-with(@src,'/')) then @src else concat($uri-folder,@src)"/>
	      <xsl:value-of select="psf:resolve-website-path($path,$website-root,$version-folder,$single,$group-folder,$siteprefix)"/>
	    </xsl:otherwise>
	    </xsl:choose>
    </xsl:attribute>
    <xsl:copy-of select="@width | @height | @alt"/>
  </graphic>

</xsl:template>
<!-- 
  Resolves pageseeder uri path to website path (uri path must be inside group-folder).
  Examples using example values below:
  
  PS uri paths:
  /ps/myproject/mygroup/website/config/header.xml
  /ps/myproject/mygroup/website/content/home.xml
  /ps/uri/123.gif
  
  Website paths for single-group='true', version-folder='':
  /config/header.xml
  /content/home.xml
  /uri/123.gif

  Website paths for single-group='true', version-folder='/2.1':
  /config/header.xml
  /content/versions/2.1/home.xml
  /uri/versions/2.1/123.gif

  Website paths for single-group='false', version-folder='':
  /myproject/mygroup/config/header.xml
  /content/myproject/mygroup/home.xml
  /myproject/mygroup/uri/123.gif

  Website paths for single-group='false', version-folder='/2.1':
  /myproject/mygroup/config/header.xml
  /content/myproject/mygroup/versions/2.1/home.xml
  /myproject/mygroup/uri/versions/2.1/123.gif
  
  @path             the PS uri path (e.g. /ps/myproject/mygroup/website/content/home.xml)
  @website-root     the website root folder in the PS group (e.g. /website)
  @version-folder   the version folder if making a version copy or '' (e.g. /2.1)
  @single-group     whether publishing as a single group to website (true|false)
  @group-folder     the PS group folder (e.g. /ps/myproject/mygroup)
  @siteprefix       the PS site prefix (e.g. /ps)
 -->
<xsl:function name="psf:resolve-website-path">
  <xsl:param name="path" />
  <xsl:param name="website-root" />
  <xsl:param name="version-folder" />
  <xsl:param name="single-group" />
  <xsl:param name="group-folder" />
  <xsl:param name="siteprefix" />
  
  <!-- get website-root-path e.g /ps/myproject/mygroup/website -->
  <xsl:variable name="website-root-path" select="concat($group-folder, $website-root)"/>

  <!-- get relative-path e.g /content/home.xml, /config/header.xml -->
  <xsl:variable name="relative-path" select="substring-after($path,$website-root-path)"/>

  <!-- get version-path e.g /versions/2.1 -->
  <xsl:variable name="version-path" select="if ($version-folder='') then '' else concat('/versions',$version-folder)" />

  <!-- get group-path e.g /myproject/mygroup -->
  <xsl:variable name="group-path" select="if ($single-group='false') then substring-after($group-folder,$siteprefix) else ''" />

  <xsl:choose>
    <!-- when a uri reference (e.g. /ps/uri/123.gif) -->
    <xsl:when test="starts-with($path,concat($siteprefix,'/uri/'))">
      <xsl:variable name="uri-relative-path" select="substring-after($path,concat($siteprefix,'/uri'))"/>
      <!-- return resolved path e.g /myproject/mygroup/uri/versions/2.1/123.gif -->
      <xsl:value-of select="concat($group-path,'/uri',$version-path,$uri-relative-path)"/>
    </xsl:when>
    <!-- when in content folder -->
	  <xsl:when test="starts-with($relative-path,'/content/')">
	    <xsl:variable name="content-relative-path" select="substring-after($path,'/content')"/>
	    <!-- return resolved path e.g /content/myproject/mygroup/versions/2.1/home.xml -->
	    <xsl:value-of select="concat('/content',$group-path,$version-path,$content-relative-path)"/>
	  </xsl:when>
	  <!-- otherwise not in content folder (don't include version) -->
	  <xsl:otherwise>
      <!-- return resolved path e.g /myproject/mygroup/config/header.xml -->
	    <xsl:value-of select="concat($group-path,$relative-path)"/>
	  </xsl:otherwise>
  </xsl:choose>

</xsl:function>
  
</xsl:stylesheet>